# Copyright (c) 2007, Robert Coup <robert.coup@onetrackmind.co.nz>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
#   1. Redistributions of source code must retain the above copyright notice,
#      this list of conditions and the following disclaimer.
#
#   2. Redistributions in binary form must reproduce the above copyright
#      notice, this list of conditions and the following disclaimer in the
#      documentation and/or other materials provided with the distribution.
#
#   3. Neither the name of Distance nor the names of its contributors may be used
#      to endorse or promote products derived from this software without
#      specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
# ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
"""
Distance and Area objects to allow for sensible and convienient calculation
and conversions.

Authors: Robert Coup, Justin Bronn, Riccardo Di Virgilio

Inspired by GeoPy (http://exogen.case.edu/projects/geopy/)
and Geoff Biggs' PhD work on dimensioned units for robotics.
"""
from measurement.base import MeasureBase, NUMERIC_TYPES, pretty_name


__all__ = [
    'Distance',
    'Area',
]


AREA_PREFIX = "sq_"


class Distance(MeasureBase):
    STANDARD_UNIT = "m"
    UNITS = {
        'chain': 20.1168,
        'chain_benoit': 20.116782,
        'chain_sears': 20.1167645,
        'british_chain_benoit': 20.1167824944,
        'british_chain_sears': 20.1167651216,
        'british_chain_sears_truncated': 20.116756,
        'british_ft': 0.304799471539,
        'british_yd': 0.914398414616,
        'clarke_ft': 0.3047972654,
        'clarke_link': 0.201166195164,
        'fathom':  1.8288,
        'ft': 0.3048,
        'german_m': 1.0000135965,
        'gold_coast_ft': 0.304799710181508,
        'indian_yd': 0.914398530744,
        'inch': 0.0254,
        'link': 0.201168,
        'link_benoit': 0.20116782,
        'link_sears': 0.20116765,
        'm': 1.0,
        'mi': 1609.344,
        'nm_uk': 1853.184,
        'rod': 5.0292,
        'sears_yd': 0.91439841,
        'survey_ft': 0.304800609601,
        'yd': 0.9144,
    }
    SI_UNITS = [
        'm'
    ]

    # Unit aliases for `UNIT` terms encountered in Spatial Reference WKT.
    ALIAS = {
        'foot': 'ft',
        'inches': 'inch',
        'meter': 'm',
        'metre': 'm',
        'mile': 'mi',
        'yard': 'yd',
        'British chain (Benoit 1895 B)': 'british_chain_benoit',
        'British chain (Sears 1922)': 'british_chain_sears',
        'British chain (Sears 1922 truncated)': (
            'british_chain_sears_truncated'
        ),
        'British foot (Sears 1922)': 'british_ft',
        'British foot': 'british_ft',
        'British yard (Sears 1922)': 'british_yd',
        'British yard': 'british_yd',
        "Clarke's Foot": 'clarke_ft',
        "Clarke's link": 'clarke_link',
        'Chain (Benoit)': 'chain_benoit',
        'Chain (Sears)': 'chain_sears',
        'Foot (International)': 'ft',
        'German legal metre': 'german_m',
        'Gold Coast foot': 'gold_coast_ft',
        'Indian yard': 'indian_yd',
        'Link (Benoit)': 'link_benoit',
        'Link (Sears)': 'link_sears',
        'Nautical Mile': 'nm',
        'Nautical Mile (UK)': 'nm_uk',
        'US survey foot': 'survey_ft',
        'U.S. Foot': 'survey_ft',
        'Yard (Indian)': 'indian_yd',
        'Yard (Sears)': 'sears_yd'
    }

    def __mul__(self, other):
        if isinstance(other, self.__class__):
            return Area(
                default_unit=AREA_PREFIX + self._default_unit,
                **{
                    AREA_PREFIX + self.STANDARD_UNIT: (
                        self.standard * other.standard
                    )
                }
            )
        elif isinstance(other, NUMERIC_TYPES):
            return self.__class__(
                default_unit=self._default_unit,
                **{self.STANDARD_UNIT: (self.standard * other)}
            )
        else:
            raise TypeError(
                '%(dst)s must be multiplied with number or %(dst)s' % {
                    "dst": pretty_name(self.__class__),
                }
            )


class Area(MeasureBase):
    STANDARD_UNIT = AREA_PREFIX + Distance.STANDARD_UNIT
    # Getting the square units values and the alias dictionary.
    UNITS = dict(
        [
            ('%s%s' % (AREA_PREFIX, k), v ** 2)
            for k, v in Distance.get_units().items()
        ]
    )
    ALIAS = dict(
        [
            (k, '%s%s' % (AREA_PREFIX, v))
            for k, v in Distance.get_aliases().items()
        ]
    )

    def __truediv__(self, other):
        if isinstance(other, NUMERIC_TYPES):
            return self.__class__(
                default_unit=self._default_unit,
                **{self.STANDARD_UNIT: (self.standard / other)}
            )
        else:
            raise TypeError(
                '%(class)s must be divided by a number' % {
                    "class": pretty_name(self)
                }
            )

    def __div__(self, other):  # Python 2 compatibility
        return type(self).__truediv__(self, other)



# -*- coding: utf-8 -*-
from measurement.base import MeasureBase


__all__ = [
    'Capacitance'
]


class Capacitance(MeasureBase):
    STANDARD_UNIT = 'F'
    UNITS = {
        'F': 1.0,
    }
    ALIAS = {
        'farad': 'F',
    }
    SI_UNITS = ['F']



from measurement.base import MeasureBase


__all__ = [
    'Energy'
]


class Energy(MeasureBase):
    STANDARD_UNIT = 'J'
    UNITS = {
        'c': 4.18400,
        'C': 4184.0,
        'J': 1.0,
    }
    ALIAS = {
        'joule': 'J',
        'calorie': 'c',
        'Calorie': 'C',
    }
    SI_UNITS = ['J', 'c']



from measurement.base import MeasureBase


__all__ = [
    'Frequency'
]


class Frequency(MeasureBase):
    STANDARD_UNIT = 'Hz'
    UNITS = {
        'Hz': 1.0,
    }
    ALIAS = {
        'hertz': 'Hz',
    }
    SI_UNITS = ['Hz']



from measurement.measures.distance import *
from measurement.measures.energy import *
from measurement.measures.temperature import *
from measurement.measures.volume import *
from measurement.measures.weight import *
from measurement.measures.speed import *
from measurement.measures.time import *
from measurement.measures.voltage import *
from measurement.measures.resistance import *
from measurement.measures.capacitance import *
from measurement.measures.frequency import *
from measurement.measures.current import *



# -*- coding: utf-8 -*-
from .base import MeasurementTestBase


from measurement.measures import Distance, Area


class DistanceTest(MeasurementTestBase):
    def test_conversion_equivalence(self):
        miles = Distance(mi=1)
        kilometers = Distance(km=1.609344)

        self.assertAlmostEqual(
            miles.km,
            kilometers.km
        )

    def test_attrib_conversion(self):
        kilometers = Distance(km=1)
        expected_meters = 1000

        self.assertAlmostEqual(
            kilometers.m,
            expected_meters
        )

    def test_identity_conversion(self):
        expected_miles = 10
        miles = Distance(mi=expected_miles)

        self.assertAlmostEqual(
            miles.mi,
            expected_miles
        )

    def test_auto_si_kwargs(self):
        meters = Distance(meter=1e6)
        megameters = Distance(megameter=1)

        self.assertEqual(
            meters,
            megameters,
        )

    def test_auto_si_attrs(self):
        one_meter = Distance(m=1)

        micrometers = one_meter.um

        self.assertEqual(
            one_meter.value * 10**6,
            micrometers
        )

    def test_area_sq_km(self):
        one_sq_km = Area(sq_km=10)
        miles_sqd = Area(sq_mi=3.8610216)

        self.assertAlmostEqual(
            one_sq_km.standard,
            miles_sqd.standard,
            places=1
        )

    def test_set_value(self):
        distance = Distance(mi=10)

        expected_standard = 16093.44
        self.assertEqual(
            distance.standard,
            expected_standard,
        )

        distance.value = 11

        expected_standard = 17702.784
        self.assertEqual(
            distance.standard,
            expected_standard
        )



from .base import MeasurementTestBase


from measurement.measures import Energy


class EnergyTest(MeasurementTestBase):
    def test_dietary_calories_kwarg(self):
        calories = Energy(Calorie=2000)
        kilojoules = Energy(kJ=8368)

        self.assertEqual(
            calories.standard,
            kilojoules.standard,
        )



# -*- coding: utf-8 -*-
from measurement.base import MeasureBase


__all__ = [
    'Resistance'
]


class Resistance(MeasureBase):
    STANDARD_UNIT = 'ohm'
    UNITS = {
        'ohm': 1.0,
    }
    ALIAS = {
    }
    SI_UNITS = ['ohm']



from measurement.base import MeasureBase


__all__ = [
    'Voltage'
]


class Voltage(MeasureBase):
    STANDARD_UNIT = 'V'
    UNITS = {
        'V': 1.0
    }
    ALIAS = {
        'volt': 'V'
    }
    SI_UNITS = ['V']



from __future__ import print_function

from measurement.base import MeasureBase, BidimensionalMeasure
from measurement.utils import get_all_measures

for measure in get_all_measures():
    classname = measure.__name__
    print(classname)
    print('-' * len(classname))
    print()
    if issubclass(measure, MeasureBase):
        units = measure.get_units()
        aliases = measure.get_aliases()
        print(
            '* *Acceptable as Arguments or Attributes*: %s' % (
                ', '.join(sorted(['``%s``' % unit for unit in units]))
            )
        )
        print(
            '* *Acceptable as Arguments*: %s' % (
                ', '.join(sorted(['``%s``' % alias for alias in aliases]))
            )
        )
    elif issubclass(measure, BidimensionalMeasure):
        print(".. note::")
        print("   This is a bi-dimensional measurement; bi-dimensional")
        print("   measures are created by finding an appropriate unit in the")
        print("   measure's primary measurement class, and an appropriate")
        print("   in the measure's reference class, and using them as a")
        print("   double-underscore-separated keyword argument (or, if")
        print("   converting to another unit, as an attribute).")
        print()
        print("   For example, to create an object representing 24 miles-per")
        print("   hour::")
        print()
        print("      >>> from measurement.measure import Speed")
        print("      >>> my_speed = Speed(mile__hour=24)")
        print("      >>> print my_speed")
        print("      24.0 mi/hr")
        print("      >>> print my_speed.km__hr")
        print("      38.624256")
        print()
        print(
            "* *Primary Measurement*: %s" % (
                measure.PRIMARY_DIMENSION.__name__
            )
        )
        print(
            "* *Reference Measurement*: %s" % (
                measure.REFERENCE_DIMENSION.__name__
            )
        )
    print()



from measurement.base import MeasureBase


__all__ = [
    'Current'
]


class Current(MeasureBase):
    STANDARD_UNIT = 'A'
    UNITS = {
        'A': 1.0,
    }
    ALIAS = {
        'amp': 'A',
        'ampere': 'A',
    }
    SI_UNITS = ['A']
